/* Copyright (c) 2006-2013 by OpenLayers Contributors (see authors.txt for
 * full list of contributors). Published under the 2-clause BSD license.
 * See license.txt in the OpenLayers distribution or repository for the
 * full text of the license. */

/**
 * @requires OpenLayers/Control/Panel.js
 * @requires OpenLayers/Control/Navigation.js
 * @requires OpenLayers/Control/DrawFeature.js
 * @requires OpenLayers/Handler/Point.js
 * @requires extend/PointWh.js
 * @requires OpenLayers/Handler/Path.js
 * @requires OpenLayers/Handler/Polygon.js
  * @requires OpenLayers/Control/SelectFeature.js
 */

/**
 * Class: OpenLayers.Control.EditingToolbar 
 * The EditingToolbar is a panel of 4 controls to draw polygons, lines, 
 * points, or to navigate the map by panning. By default it appears in the 
 * upper right corner of the map.
 * 
 * Inherits from:
 *  - <OpenLayers.Control.Panel>
 */
EditingToolbarWh = OpenLayers.Class(
  OpenLayers.Control.Panel, {
    /**
     * APIProperty: citeCompliant
     * {Boolean} If set to true, coordinates of features drawn in a map extent
     * crossing the date line won't exceed the world bounds. Default is false.
     */
    citeCompliant: false,

    /**
     * Constructor: OpenLayers.Control.EditingToolbar
     * Create an editing toolbar for a given layer. 
     *
     * Parameters:
     * layer - {<OpenLayers.Layer.Vector>} 
     * options - {Object} 
     */
    initialize: function(layer, options) {
		//alert('there1');
        OpenLayers.Control.Panel.prototype.initialize.apply(this, [options]);
        //alert('there2');
		//navigation control
        this.addControls(
          [ new OpenLayers.Control.Navigation() ]
        );  
        var controls = [
            /*new OpenLayers.Control.DrawFeature(layer, OpenLayers.Handler.Point, {
                displayClass: 'olControlDrawFeaturePointWh',
                handlerOptions: {citeCompliant: this.citeCompliant}
            })*/
			
            new OpenLayers.Control.DrawFeature(layer, OpenLayers.Handler.Path, {
                displayClass: 'olControlDrawFeaturePathWh',
                handlerOptions: {citeCompliant: this.citeCompliant}
            })
            ,new OpenLayers.Control.DrawFeature(layer, OpenLayers.Handler.Polygon, {
                displayClass: 'olControlDrawFeaturePolygonWh',
                handlerOptions: {citeCompliant: this.citeCompliant}
            }),new OpenLayers.Control.DrawFeature(layer, PointWh, {
                displayClass: 'olControlDrawFeaturePointWh',
                handlerOptions: {citeCompliant: this.citeCompliant}
            })
        ];
		//alert('there3');
        this.addControls(controls);
		//----------select----
		//alert("vlayer"+vlayer);
		//alert("map"+map);
		var select = new OpenLayers.Control.SelectFeature([vlayer]);
		map.addControl(select);
		select.activate();
		//alert("select activate");
		//------dialog------
		var dialog;
		vlayer.events.on({
			featureselected: function(event) {
								//alert("here featureselected");
								var feature = event.feature;
								var area = feature.geometry.getArea();
								var id = feature.attributes.key;
								var output = "Building: " + id + " Area: " + area.toFixed(2);
									dialog = $("<div title='Feature Info'>" + output + "</div>").dialog();
			},
			featureunselected: function() {
				dialog.dialog("destroy").remove();
			}
		});
		//-----------
    },

    /**
     * Method: draw
     * calls the default draw, and then activates mouse defaults.
     *
     * Returns:
     * {DOMElement}
     */
    draw: function() {
        var div = OpenLayers.Control.Panel.prototype.draw.apply(this, arguments);
        if (this.defaultControl === null) {
            this.defaultControl = this.controls[0];
        }
        return div;
    },

    CLASS_NAME: "EditingToolbarWh"
});    
